@@ -1,5 +1,5 @@
 package AnyEvent::XMPP;
-use warnings;
+no warnings;
 use strict;
 
 =head1 NAME
@@ -8,11 +8,11 @@ AnyEvent::XMPP - An implementation of the XMPP Protocol
 
 =head1 VERSION
 
-Version 0.2
+Version 0.3
 
 =cut
 
-our $VERSION = '0.2';
+our $VERSION = '0.3';
 
 =head1 SYNOPSIS
 
@@ -118,6 +118,11 @@ Here are some notes to the last releases (release of this version is at top):
 
 =over 4
 
+=item * 0.3
+
+Fixed some small bugs and improved documentation a bit, especially w.r.t.
+parameter passing of host and ports.
+
 =item * 0.2
 
 Renamed module from L<Net::XMPP2> to L<AnyEvent::XMPP>. L<Net::XMPP2> is herby
